//
// count.cxx
// kkyoda@riken.jp
//

#define Pi 3.14159265

#include <memory>   // std::auto_ptr
#include <iostream>
#include <cstring>
#include <cmath>
#include <unistd.h>
#include <limits.h>
#include <list>
#include <sstream>

#include "bdml0.15.hxx"
#include "xyzt.h"

using namespace std;

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " *.bdml0.15.xml" << endl;
    return 1;
  }

  try {
    auto_ptr<bdml::bdmlDocument> h (bdml::bdml (argv[1]));

    list<xyzt> xyztl;
    xyztl.clear();

    stringstream ss;

    double xscale = 0.0;
    xscale = h->data().scaleUnit().xScale();
    double yscale = 0.0;
    yscale = h->data().scaleUnit().yScale();
    double zscale = 0.0;
    zscale = h->data().scaleUnit().zScale();
    string xyzunit;
    xyzunit = h->data().scaleUnit().xyzUnit();
    double tscale = 0.0;
    tscale = h->data().scaleUnit().tScale();
    string tunit;
    tunit = h->data().scaleUnit().tUnit();

    for (bdml::Data::component_const_iterator p = h->data().component().begin(); p != h->data().component().end(); p++) {
      xyzt tmp_xyzt(1.0, 1.0, 1.0, p->time());
      xyztl.push_back(tmp_xyzt);
    }

    xyztl.sort();
    list<xyzt>::iterator v;

    int st = 0;
    int et = 0;
    v = xyztl.begin();
    st = v->getT();
    for (;v != xyztl.end(); v++) {
      et = v->getT();
    }

//    cout << "time interval: " << tscale << " " << tunit << "(s)" << endl;
//    cout << "#t\tcount" << endl;
    int count = 0;
    for (int i = st; i <= et; i++) {
      count = 0;
      for (v = xyztl.begin(); v != xyztl.end(); v++) {
        if (v->getT() == i) {
          count++;
        }
      }
      cout << i << "\t" << count << endl;
    }
  }

  catch (const xml_schema::exception& e)
  {
    cerr << e << endl;
    return 1;
  }

}
