//
// move.cxx
// kkyoda@riken.jp
//

#define Pi 3.14159265

#include <memory>   // std::auto_ptr
#include <iostream>
#include <cstring>
#include <cmath>
#include <unistd.h>
#include <limits.h>
#include <list>
#include <sstream>

#include "bdml0.15.hxx"
#include "xyzt.h"

using namespace std;

int
main (int argc, char* argv[])
{
  if (argc != 3)
  {
    cerr << "usage: " << argv[0] << " *.bdml0.15.xml componentName" << endl;
    return 1;
  }

  try {
    auto_ptr<bdml::bdmlDocument> h (bdml::bdml (argv[1]));

    string comp = argv[2];

    double gx;
    double gy;
    double gz;
    double area;
    double total;

    list<xyzt> xyztl;
    xyztl.clear();

    stringstream ss;
    int flag = 0;

    double xscale = 0.0;
    xscale = h->data().scaleUnit().xScale();
    double yscale = 0.0;
    yscale = h->data().scaleUnit().yScale();
    double zscale = 0.0;
    zscale = h->data().scaleUnit().zScale();
    string xyzunit;
    xyzunit = h->data().scaleUnit().xyzUnit();

    for (bdml::Data::component_const_iterator p = h->data().component().begin(); p != h->data().component().end(); p++) {
      if (!strcmp(p->componentName()->c_str(), comp.c_str())) {
        gx = gy = gz = area = total = 0.0;
        for (bdml::Component::measurement_const_iterator r = p->measurement().begin(); r != p->measurement().end(); r++) {
          if (r->point()) {
            for (bdml::Point::coords_const_iterator q = r->point()->coords().begin(); q != r->point()->coords().end(); q++) {
              string str;
              int l;
              double x, y, z;
              l = str.find(",");
              x = atof(str.substr(0, l).c_str());
              str = str.substr(l + 1);
              l = str.find(",");
              y = atof(str.substr(0, l).c_str());
              str = str.substr(l + 1);
              z = atof(str.c_str());
              gx += x;
              gy += y;
              gz += z;
              total += 1.0;
            }
            flag = 1;
          } else if (r->line()) {
            for (bdml::Line::coords_const_iterator q = r->line()->coords().begin(); q != r->line()->coords().end(); q++) {
              string str;
              int l;
              double x = 0.0, y = 0.0, z = 0.0;
              double prev_x, prev_y;
              int sflag = 0;
              double startx = 0.0, starty = 0.0, startz = 0.0;
              int cnt = 0;
              ss << (*q);
              while (ss >> str) {
                cnt++;
                prev_x = x;
                prev_y = y;
                l = str.find(",");
                x = atof(str.substr(0, l).c_str());
                str = str.substr(l + 1);
                l = str.find(",");
                y = atof(str.substr(0, l).c_str());
                str = str.substr(l + 1);
                z = atof(str.c_str());
                if (sflag == 0) {
                  startx = x;
                  starty = y;
                  startz = z;
                  sflag = 1;
                }
              }

              if (startx == x && starty == y && startz == z) {
                ss.clear();
                ss << (*q);

                double subarea = 0.0;
                for (int i = 0; i < cnt - 1; i++) {
                  ss >> str;
                  l = str.find(",");
                  x = atof(str.substr(0, l).c_str());
                  str = str.substr(l + 1);
                  l = str.find(",");
                  y = atof(str.substr(0, l).c_str());
                  str = str.substr(l + 1);
                  z = atof(str.c_str());
                  gx += (prev_x + x) * (prev_x * y - x * prev_y);
                  gy += (prev_y + y) * (prev_x * y - x * prev_y);
                  subarea += prev_x * y - x * prev_y;

                  prev_x = x;
                  prev_y = y;
                }
                ss >> str;
                subarea = subarea / 2.0;
                area += subarea;
                gz += z * subarea; 

                flag = 2;
              } else {
                ss.clear();
                ss << (*q);
                while (ss >> str) {
                  int l;
                  double x, y, z;
                  l = str.find(",");
                  x = atof(str.substr(0, l).c_str());
                  str = str.substr(l + 1);
                  l = str.find(",");
                  y = atof(str.substr(0, l).c_str());
                  str = str.substr(l + 1);
                  z = atof(str.c_str());
                  gx += x;
                  gy += y;
                  gz += z;
                  total += 1.0;
                }

                flag = 3;
              }
              ss.clear();
            }

          } else if (r->face()) {
            // should be implemented in the near future
            flag = 4;

          } else if (r->circle()) {
            string str = r->circle()->coords();
            double rad = r->circle()->radius();
            int l;
            double x, y, z;
            l = str.find(",");
            x = atof(str.substr(0, l).c_str());
            str = str.substr(l + 1);
            l = str.find(",");
            y = atof(str.substr(0, l).c_str());
            str = str.substr(l + 1);
            z = atof(str.c_str());
            gx += x * Pi * rad * rad;
            gy += y * Pi * rad * rad;
            gz += z * Pi * rad * rad;
            total += Pi * rad * rad;

            flag = 5;

          } else if (r->sphere()) {
            string str = r->sphere()->coords();
            double rad = r->sphere()->radius();
            int l;
            double x, y, z;
            l = str.find(",");
            x = atof(str.substr(0, l).c_str());
            str = str.substr(l + 1);
            l = str.find(",");
            y = atof(str.substr(0, l).c_str());
            str = str.substr(l + 1);
            z = atof(str.c_str());
            gx += x * 4.0 * Pi * rad * rad / 3.0;
            gy += y * 4.0 * Pi * rad * rad / 3.0;
            gz += z * 4.0 * Pi * rad * rad / 3.0;
            total += 4.0 * Pi * rad * rad / 3.0;

            flag = 6;
          }
        }

        double xx, yy, zz;
        if (flag == 1) {  // point
          xx = gx / total * xscale;
          yy = gy / total * yscale;
          zz = gz / total * zscale;

          xyzt tmp_xyzt(xx, yy, zz, p->time());
          xyztl.push_back(tmp_xyzt);
        } else if (flag == 2) {  // line (closed)
          xx = gx / 6.0 / area * xscale;
          yy = gy / 6.0 / area * yscale;
          zz = gz / area * zscale;

          xyzt tmp_xyzt(xx, yy, zz, p->time());
          xyztl.push_back(tmp_xyzt);
        } else if (flag == 3) {  // line
          xx = gx / total * xscale;
          yy = gy / total * yscale;
          zz = gz / total * zscale;

          xyzt tmp_xyzt(xx, yy, zz, p->time());
          xyztl.push_back(tmp_xyzt);
        } else if (flag == 4) {  // face
          cout << "this program cannot handle the Face structure." << endl;
          exit(-1);
        } else if (flag == 5) {  // circle
          xx = gx / total * xscale;
          yy = gy / total * yscale;
          zz = gz / total * zscale;

          xyzt tmp_xyzt(xx, yy, zz, p->time());
          xyztl.push_back(tmp_xyzt);
        } else if (flag == 6) {  // sphere
          xx = gx / total * xscale;
          yy = gy / total * yscale;
          zz = gz / total * zscale;

          xyzt tmp_xyzt(xx, yy, zz, p->time());
          xyztl.push_back(tmp_xyzt);
        } else {  // other
          cout << "this program can handle only Point, Line, Circle, and Sphere structure." << endl;
          exit(-1);
        }
      }
    }

    if (xyztl.size() == 0) {
      cout << "no component: " << comp << endl;
      exit(-1);
    }

    xyztl.sort();
    list<xyzt>::iterator v;
    cout << "#componentName: " << comp << ", Unit: " << xyzunit << endl;
    cout << "#movement" << endl;

    double distance = 0.0;
    v = xyztl.begin();
    double px = v->getX();
    double py = v->getY();
    double pz = v->getZ();
    v++;
    for (; v != xyztl.end(); v++) {
      distance += sqrt((v->getX() - px) * (v->getX() - px) + (v->getY() - py) * (v->getY() - py) + (v->getZ() - pz) * (v->getZ() - pz));
      px = v->getX();
      py = v->getY();
      pz = v->getZ();
    }
    cout << distance << endl;
  }

  catch (const xml_schema::exception& e)
  {
    cerr << e << endl;
    return 1;
  }

}
