//
// timelength.cxx
// kkyoda@riken.jp
//

#include <memory>   // std::auto_ptr
#include <iostream>
#include <sstream>
#include <cstring>
#include <unistd.h>
#include <limits.h>
#include "bdml0.15.hxx"
using namespace std;

//stringstream ss;

int main(int argc, char* argv[])
{
  if (argc != 3)
  {
    cerr << "usage: " << argv[0] << " *.bdml0.15.xml" << " componentName" << endl;
    return 1;
  }

  string compname = argv[2];

  int mintime = +32767;
  int maxtime = -32768;

  try {
    auto_ptr<bdml::bdmlDocument> h (bdml::bdml (argv[1]));

    double tscale;
    tscale = h->data().scaleUnit().tScale();
    string tunit;
    tunit = h->data().scaleUnit().tUnit();
    int flag = 0;

    for (bdml::Data::component_const_iterator p = h->data().component().begin(); p != h->data().component().end(); p++) {
      if (!strcmp(p->componentName()->c_str(), compname.c_str())) {
        if (mintime > p->time()) {
          mintime = p->time();
        } else if (maxtime < p->time()) {
          maxtime = p->time();
        }
        flag = 1;
      }
    }

    if (flag == 1) {
      cout << "#componentName: " << compname << ", Unit: " << tunit << endl;
      cout << "#time length" << endl; 
      if ((maxtime - (mintime - 1)) * tscale == 1.0) {
        cout << (double) (maxtime - (mintime - 1)) * tscale << endl;
      } else {
        cout << (double) (maxtime - (mintime - 1)) * tscale << endl;
      }
    } else {
      cout << "no componentName: " << compname << endl;
    }
  }
  catch (const xml_schema::exception& e)
  {
    cerr << e << endl;
    return 1;
  }

}
