//
// volume.cxx
// kkyoda@riken.jp
//

#define Pi 3.14159265

#include <memory>   // std::auto_ptr
#include <iostream>
#include <cstring>
#include <cmath>
#include <unistd.h>
#include <limits.h>
#include <list>
#include <sstream>

#include "bdml0.15.hxx"
#include "vt.h"

using namespace std;

int
main (int argc, char* argv[])
{
  if (argc != 4)
  {
    cerr << "usage: " << argv[0] << " *.bdml0.15.xml -{id, name} component{ID, Name}" << endl;
    return 1;
  }

  try {
    auto_ptr<bdml::bdmlDocument> h (bdml::bdml (argv[1]));

    int mode = 0;
    string comp;
    string smode = argv[2];
    if (!strcmp(smode.c_str(), "-id")) {
      mode = 1;
      comp = argv[3];
    } else if (!strcmp(smode.c_str(), "-name")) {
      mode = 2;
      comp = argv[3];
    }

    double area;
    double vol;
    double length;
    double total;

    list<vt> vtl;
    vtl.clear();

    stringstream ss;
    int flag = 0;

    double xscale = 0.0;
    xscale = h->data().scaleUnit().xScale();
    double yscale = 0.0;
    yscale = h->data().scaleUnit().yScale();
    double zscale = 0.0;
    zscale = h->data().scaleUnit().zScale();
    string xyzunit;
    xyzunit = h->data().scaleUnit().xyzUnit();
    double tscale = 0.0;
    tscale = h->data().scaleUnit().tScale();
    string tunit;
    tunit = h->data().scaleUnit().tUnit();

    for (bdml::Data::component_const_iterator p = h->data().component().begin(); p != h->data().component().end(); p++) {
      int compflag = 0;
      if (mode == 1) {
        if (!strcmp(p->componentID().c_str(), comp.c_str())) {
          compflag = 1;
        }
      } else if (mode == 2) {
        if (!strcmp(p->componentName()->c_str(), comp.c_str())) {
          compflag = 1;
        }
      }

      if (compflag == 1) {
        area = total = vol = length = 0.0;
        for (bdml::Component::measurement_const_iterator r = p->measurement().begin(); r != p->measurement().end(); r++) {
          if (r->point()) {
            flag = 1;
          } else if (r->line()) {
            for (bdml::Line::coords_const_iterator q = r->line()->coords().begin(); q != r->line()->coords().end(); q++) {
              string str;
              int l;
              double x = 0.0, y = 0.0, z = 0.0;
              double prev_x, prev_y;
              int sflag = 0;
              double startx = 0.0, starty = 0.0, startz = 0.0;
              int cnt = 0;
              ss << (*q);
              while (ss >> str) {
                cnt++;
                prev_x = x;
                prev_y = y;
                l = str.find(",");
                x = atof(str.substr(0, l).c_str());
                str = str.substr(l + 1);
                l = str.find(",");
                y = atof(str.substr(0, l).c_str());
                str = str.substr(l + 1);
                z = atof(str.c_str());
                if (sflag == 0) {
                  startx = x;
                  starty = y;
                  startz = z;
                  sflag = 1;
                }
              }

              if (startx == x && starty == y && startz == z) {
                ss.clear();
                ss << (*q);

                double subarea = 0.0;
                for (int i = 0; i < cnt - 1; i++) {
                  ss >> str;
                  l = str.find(",");
                  x = atof(str.substr(0, l).c_str());
                  str = str.substr(l + 1);
                  l = str.find(",");
                  y = atof(str.substr(0, l).c_str());
                  str = str.substr(l + 1);
                  z = atof(str.c_str());
                  subarea += prev_x * y - x * prev_y;

                  prev_x = x;
                  prev_y = y;
                }
                ss >> str;
                subarea = subarea / 2.0;
                area += subarea;

                flag = 2;
              } else {
                ss.clear();
                ss << (*q);
                ss >> str;
                int l;
                double pxx, pyy, pzz;
                l = str.find(",");
                pxx = atof(str.substr(0, l).c_str());
                str = str.substr(l + 1);
                l = str.find(",");
                pyy = atof(str.substr(0, l).c_str());
                str = str.substr(l + 1);
                pzz = atof(str.c_str());
                while (ss >> str) {
                  double x, y, z;
                  l = str.find(",");
                  x = atof(str.substr(0, l).c_str());
                  str = str.substr(l + 1);
                  l = str.find(",");
                  y = atof(str.substr(0, l).c_str());
                  str = str.substr(l + 1);
                  z = atof(str.c_str());
                  length += sqrt((x - pxx) * (x - pxx) + (y - pyy) * (y - pyy) + (z - pzz) * (z - pzz));
                  pxx = x;
                  pyy = y;
                  pzz = z;
                }

                flag = 3;
              }
              ss.clear();
            }

          } else if (r->face()) {
            // should be implemented in the near future
            flag = 4;

          } else if (r->circle()) {
            double rad = r->circle()->radius();
            area += Pi * rad * rad;

            flag = 5;

          } else if (r->sphere()) {
            double rad = r->sphere()->radius();
            vol += 4.0 * Pi * rad * rad / 3.0;

            flag = 6;
          }
        }

        double val;
        if (flag == 1) {  // point
          cerr << "calculation was not executed: point" << endl;
          exit(-1);
        } else if (flag == 2) {  // line (closed)
          if (zscale != 0.0)
            val = abs(area) * xscale * yscale * zscale;
          else
            val = abs(area) * xscale * yscale;

          vt tmp_vt(val, p->time());
          vtl.push_back(tmp_vt);
        } else if (flag == 3) {  // line
          val = length;

          vt tmp_vt(val, p->time());
          vtl.push_back(tmp_vt);
        } else if (flag == 4) {  // face
          cout << "this program cannot handle the Face structure." << endl;
          exit(-1);
        } else if (flag == 5) {  // circle
          val = area; 

          vt tmp_vt(val, p->time());
          vtl.push_back(tmp_vt);
        } else if (flag == 6) {  // sphere
          val = vol;

          vt tmp_vt(val, p->time());
          vtl.push_back(tmp_vt);
        } else {  // other
          cout << "this program can handle only Line, Circle, and Sphere structure." << endl;
          exit(-1);
        }
      }
    }

    vtl.sort();
    list<vt>::iterator v;

    if (vtl.size() == 0) {
      cerr << "no component: " << comp << endl;
      exit(-1);
    }

    if (mode == 1) {
      cout << "#componentID: " << comp;
    } else if (mode == 2) {
      cout << "#componentName: " << comp;
    }
    if (flag == 2) {
      if (zscale == 0.0) {
        cout << ", Unit: " << xyzunit << "^2, time interval: " << tscale << " " << tunit << "(s)" << endl;
        cout << "#t\tvolume" << endl;
        for (v = vtl.begin(); v != vtl.end(); v++) {
          cout << v->getT() << "\t" << v->getV() << endl;
        }
      } else {
        cout << ", Unit: " << xyzunit << "^3, time interval: " << tscale << " " << tunit << "(s)" << endl;
        cout << "#t\tvolume" << endl;
        for (v = vtl.begin(); v != vtl.end(); v++) {
          cout << v->getT() << "\t" << v->getV() << endl;
        }
      }
    } else if (flag == 3) {
      cout << ", Unit: " << xyzunit << ", time interval: " << tscale << " " << tunit << "(s)" << endl;
      cout << "#t\tlength" << endl;
      for (v = vtl.begin(); v != vtl.end(); v++) {
        cout << v->getT() << "\t" << v->getV() << endl;
      }
    } else if (flag == 5) {
      cout << ", Unit: " << xyzunit << "^2, time interval: " << tscale << " " << tunit << "(s)" << endl;
      cout << "#t\tarea" << endl;
      for (v = vtl.begin(); v != vtl.end(); v++) {
        cout << v->getT() << "\t" << v->getV() << endl;
      }
    } else if (flag == 6) {
      cout << ", Unit: " << xyzunit << "^3, time interval: " << tscale << " " << tunit << "(s)" << endl;
      cout << "#t\tvolume" << endl;
      for (v = vtl.begin(); v != vtl.end(); v++) {
        cout << v->getT() << "\t" << v->getV() << endl;
      }
    }
  }

  catch (const xml_schema::exception& e)
  {
    cerr << e << endl;
    return 1;
  }

}
